'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var yargs = require('yargs');
var fs = require('fs-extra');
var envDotProp = require('env-dot-prop');
var humanize = _interopDefault(require('humanize-string'));
var titleize = _interopDefault(require('titleize'));
var fp = require('lodash/fp');
var fs$1 = require('fs');
var path = require('path');
var resolve = require('resolve');
var logger = require('signale');
var logger__default = _interopDefault(logger);
var PrettyError = _interopDefault(require('pretty-error'));
var tslib_1 = require('tslib');
var mdast = require('docz-utils/lib/mdast');
var fs$2 = require('docz-utils/lib/fs');
var glob = _interopDefault(require('fast-glob'));
var crypto = require('crypto');
var slugify = _interopDefault(require('@sindresorhus/slugify'));
var pReduce = _interopDefault(require('p-reduce'));
var getPkgRepo = _interopDefault(require('get-pkg-repo'));
var findup = _interopDefault(require('find-up'));
var WS = _interopDefault(require('ws'));
var loadCfg = require('load-cfg');
var detectPort = _interopDefault(require('detect-port'));
var Config = _interopDefault(require('webpack-chain'));
var matter = _interopDefault(require('remark-frontmatter'));
var slug = _interopDefault(require('rehype-slug'));
var remarkDocz = _interopDefault(require('remark-docz'));
var rehypeDocz = _interopDefault(require('rehype-docz'));
var webpack = require('webpack');
var webpack__default = _interopDefault(webpack);
var webpackBarPlugin = _interopDefault(require('webpackbar'));
var friendlyErrorsPlugin = _interopDefault(require('friendly-errors-webpack-plugin'));
var htmlMinifier = require('html-minifier');
var miniHtmlWebpack = require('mini-html-webpack-plugin');
var miniHtmlWebpack__default = _interopDefault(miniHtmlWebpack);
var manifestPlugin = _interopDefault(require('webpack-manifest-plugin'));
var watchMissingNodeModules = _interopDefault(require('react-dev-utils/WatchMissingNodeModulesPlugin'));
require('react-dev-utils/ModuleNotFoundPlugin');
var webpackBundleAnalyzer = require('webpack-bundle-analyzer');
var ctags = require('common-tags');
var TerserPlugin = require('terser-webpack-plugin');
var merge = _interopDefault(require('deepmerge'));
var getCacheIdentifier = _interopDefault(require('react-dev-utils/getCacheIdentifier'));
var chalk = _interopDefault(require('chalk'));
var WebpackDevServer = _interopDefault(require('webpack-dev-server'));
var express = require('express');
var errorOverlayMiddleware = _interopDefault(require('react-dev-utils/errorOverlayMiddleware'));
var evalSourceMapMiddleware = _interopDefault(require('react-dev-utils/evalSourceMapMiddleware'));
var ignoredFiles = _interopDefault(require('react-dev-utils/ignoredFiles'));
var chokidar = _interopDefault(require('chokidar'));
var equal = _interopDefault(require('fast-deep-equal'));
var get = _interopDefault(require('lodash/get'));
var externalProptypesHandler = _interopDefault(require('react-docgen-external-proptypes-handler'));
var importedProptypesHandler = _interopDefault(require('react-docgen-imported-proptype-handler'));
var actualNameHandler = _interopDefault(require('react-docgen-actual-name-handler'));
var reactDocgenTs = _interopDefault(require('react-docgen-typescript'));
var reactDocgen = _interopDefault(require('react-docgen'));

const ensureSlash = (filepath, needsSlash) => {
    const hasSlash = filepath.endsWith('/');
    if (hasSlash && !needsSlash) {
        return filepath.substr(filepath, filepath.length - 1);
    }
    else if (!hasSlash && needsSlash) {
        return `${filepath}/`;
    }
    else {
        return filepath;
    }
};
const root = fs$1.realpathSync(process.cwd());
const resolveApp = (to) => path.resolve(root, to);
const resolveOwn = (to) => path.resolve(__dirname, '../', to);
const templates = path.join(resolve.sync('docz-core'), '../templates');
const packageJson = resolveApp('package.json');
const servedPath = (base) => ensureSlash(base, true);
const docz = resolveApp('.docz');
const app = path.resolve(docz, 'app/');
const cache = path.resolve(docz, 'cache/');
const appPublic = path.resolve(docz, 'public/');
const appNodeModules = resolveApp('node_modules');
const appPackageJson = resolveApp('package.json');
const appYarnLock = resolveOwn('yarn.lock');
const ownNodeModules = resolveOwn('node_modules');
const getDist = (dest) => path.join(root, dest);
const distPublic = (dest) => path.join(dest, 'public/');
const importsJs = path.resolve(app, 'imports.js');
const rootJs = path.resolve(app, 'root.jsx');
const indexJs = path.resolve(app, 'index.jsx');
const indexHtml = path.resolve(app, 'index.html');
const db = path.resolve(app, 'db.json');

var paths = /*#__PURE__*/Object.freeze({
  root: root,
  resolveApp: resolveApp,
  resolveOwn: resolveOwn,
  templates: templates,
  packageJson: packageJson,
  servedPath: servedPath,
  docz: docz,
  app: app,
  cache: cache,
  appPublic: appPublic,
  appNodeModules: appNodeModules,
  appPackageJson: appPackageJson,
  appYarnLock: appYarnLock,
  ownNodeModules: ownNodeModules,
  getDist: getDist,
  distPublic: distPublic,
  importsJs: importsJs,
  rootJs: rootJs,
  indexJs: indexJs,
  indexHtml: indexHtml,
  db: db
});

const getEnv = (val, defaultValue = null) => envDotProp.get(val, defaultValue, { parse: true });
const getInitialTitle = (pkg) => {
    const name = fp.get('name', pkg) || 'MyDoc';
    return titleize(humanize(name.replace(/^@.*\//, '')));
};
const getInitialDescription = (pkg) => fp.get('description', pkg) || 'My awesome app using docz';
const setArgs = (yargs$$1) => {
    const pkg = fs.readJsonSync(appPackageJson, { throws: false });
    return yargs$$1
        .option('base', {
        type: 'string',
        default: getEnv('docz.base', '/'),
    })
        .option('source', {
        alias: 'src',
        type: 'string',
        default: getEnv('docz.source', './'),
    })
        .option('files', {
        type: 'string',
        default: getEnv('docz.files', '**/*.mdx'),
    })
        .option('ignore', {
        type: 'array',
        default: getEnv('docz.ignore', []),
    })
        .option('public', {
        type: 'string',
        default: getEnv('docz.public', '/public'),
    })
        .option('dest', {
        alias: 'd',
        type: 'string',
        default: getEnv('docz.dest', '.docz/dist'),
    })
        .option('editBranch', {
        alias: 'eb',
        type: 'string',
        default: getEnv('docz.edit.branch', 'master'),
    })
        .option('config', {
        type: 'string',
        default: getEnv('docz.config', ''),
    })
        .option('title', {
        type: 'string',
        default: getEnv('docz.title', getInitialTitle(pkg)),
    })
        .option('description', {
        type: 'string',
        default: getEnv('docz.description', getInitialDescription(pkg)),
    })
        .option('theme', {
        type: 'string',
        default: getEnv('docz.theme', 'docz-theme-default'),
    })
        .option('typescript', {
        alias: 'ts',
        type: 'boolean',
        default: getEnv('docz.typescript', false),
    })
        .option('propsParser', {
        type: 'boolean',
        default: getEnv('docz.props.parser', true),
    })
        .option('wrapper', {
        type: 'string',
        default: getEnv('docz.wrapper', null),
    })
        .option('indexHtml', {
        type: 'string',
        default: getEnv('docz.index.html', null),
    })
        .option('ordering', {
        type: 'string',
        default: getEnv('docz.ordering', 'descending'),
    })
        .option('debug', {
        type: 'boolean',
        default: getEnv('docz.debug', false),
    })
        .option('host', {
        type: 'string',
        default: getEnv('docz.host', '127.0.0.1'),
    })
        .option('port', {
        alias: 'p',
        type: 'number',
        default: getEnv('docz.port', 3000),
    })
        .option('websocketHost', {
        type: 'string',
        default: getEnv('docz.websocket.host', '127.0.0.1'),
    })
        .option('websocketPort', {
        type: 'number',
        default: getEnv('docz.websocket.port', 60505),
    })
        .option('hashRouter', {
        type: 'boolean',
        default: getEnv('docz.hash.router', false),
    })
        .option('native', {
        type: 'boolean',
        default: getEnv('docz.native', false),
    })
        .option('codeSandbox', {
        type: 'boolean',
        default: getEnv('docz.codeSandbox', true),
    })
        .option('sourcemaps', {
        type: 'boolean',
        default: getEnv('docz.sourcemaps', true),
    });
};

const populateNodePath = () => {
    // We support resolving modules according to `NODE_PATH`.
    // It works similar to `NODE_PATH` in Node itself:
    // https://nodejs.org/api/modules.html#modules_loading_from_the_global_folders
    // Note that unlike in Node, only *relative* paths from `NODE_PATH` are honored.
    // Otherwise, we risk importing Node.js core modules into an app instead of Webpack shims.
    // https://github.com/facebook/create-react-app/issues/1023#issuecomment-265344421
    // We also resolve them to make sure all tools using them work consistently.
    envDotProp.set('node.path', envDotProp.get('node.path', '')
        .split(path.delimiter)
        .filter((folder) => folder && !path.isAbsolute(folder))
        .map((folder) => path.resolve(root, folder))
        .join(path.delimiter));
};
const configDotEnv = () => {
    const NODE_ENV = envDotProp.get('node.env');
    const dotenv = resolveApp('.env');
    const dotenvFiles = [
        `${dotenv}.${NODE_ENV}.local`,
        `${dotenv}.${NODE_ENV}`,
        // Don't include `.env.local` for `test` environment
        // since normally you expect tests to produce the same
        // results for everyone
        NODE_ENV !== 'test' && `${dotenv}.local`,
        dotenv,
    ];
    // Load environment variables from .env* files. Suppress warnings using silent
    // if this file is missing. dotenv will never modify any environment variables
    // that have already been set.  Variable expansion is supported in .env files.
    // https://github.com/motdotla/dotenv
    dotenvFiles.filter(Boolean).forEach(dotenvFile => {
        require('dotenv').config({
            path: dotenvFile,
        });
    });
};
const setEnv = (env) => {
    envDotProp.set('babel.env', env);
    envDotProp.set('node.env', env);
    configDotEnv();
    populateNodePath();
};
const getClientEnvironment = (publicUrl) => {
    const APP_TEST = /^(REACT_APP_)|(ANGULAR_APP_)|(VUE_APP_)|(DOCZ_)/i;
    const raw = Object.keys(process.env)
        .filter(key => APP_TEST.test(key))
        .reduce((env, key) => {
        env[key] = process.env[key];
        return env;
    }, {
        // Useful for determining whether we’re running in production mode. Most
        // importantly, it switches React into the correct mode.
        NODE_ENV: envDotProp.get('node.env') || 'development',
        // Useful for resolving the correct path to static assets in `public`. For
        // example, <img src={process.env.PUBLIC_URL + '/img/logo.png'} />. This should
        // only be used as an escape hatch. Normally you would put images into the `src`
        // and `import` them in code to get their
        PUBLIC_URL: publicUrl,
    });
    const stringified = {
        'process.env': Object.keys(raw).reduce((env, key) => {
            env[key] = JSON.stringify(raw[key]);
            return env;
        }, {}),
    };
    return { raw, stringified };
};

const createId = (file) => crypto.createHash('md5')
    .update(file)
    .digest('hex');
class Entry {
    constructor(ast, file, src) {
        const filepath = this.getFilepath(file, src);
        const parsed = mdast.getParsedData(ast);
        const name = this.getName(filepath, parsed);
        this.id = createId(file);
        this.filepath = filepath;
        this.link = null;
        this.slug = this.slugify(filepath);
        this.route = this.getRoute(parsed);
        this.name = name;
        this.order = parsed.order || 0;
        this.menu = parsed.menu || null;
        this.headings = mdast.headingsFromAst(ast);
        this.settings = parsed;
    }
    setLink(url) {
        if (url) {
            this.link = url.replace('{{filepath}}', this.filepath);
        }
    }
    getFilepath(file, src) {
        const srcPath = path.resolve(root, src);
        const filepath = path.relative(srcPath, file);
        if (process.platform === 'win32') {
            return filepath.split('\\').join('/');
        }
        return filepath;
    }
    getName(filepath, parsed) {
        const filename = humanize(path.parse(filepath).name);
        return parsed && parsed.name ? parsed.name : filename;
    }
    slugify(filepath) {
        const ext = path.extname(filepath);
        const fileWithoutExt = filepath.replace(ext, '');
        return slugify(fileWithoutExt);
    }
    getRoute(parsed) {
        return parsed && parsed.route ? parsed.route : `/${this.slug}`;
    }
}

class Plugin {
    constructor(p) {
        this.setConfig = p.setConfig;
        this.modifyBundlerConfig = p.modifyBundlerConfig;
        this.modifyBabelRc = p.modifyBabelRc;
        this.modifyFiles = p.modifyFiles;
        this.onPreCreateApp = p.onPreCreateApp;
        this.onCreateWebpackChain = p.onCreateWebpackChain;
        this.onCreateApp = p.onCreateApp;
        this.onServerListening = p.onServerListening;
        this.onPreBuild = p.onPreBuild;
        this.onPostBuild = p.onPostBuild;
        this.onPreRender = p.onPreRender;
        this.onPostRender = p.onPostRender;
    }
    static runPluginsMethod(plugins) {
        return (method, ...args) => {
            if (plugins && plugins.length > 0) {
                for (const plugin of plugins) {
                    const fn = fp.get(method, plugin);
                    fp.isFunction(fn) && fn(...args);
                }
            }
        };
    }
    static propsOfPlugins(plugins) {
        return prop => plugins && plugins.length > 0
            ? plugins.map(p => fp.get(prop, p)).filter(Boolean)
            : [];
    }
    static reduceFromPlugins(plugins) {
        return (method, initial, ...args) => {
            return [...(plugins || [])].reduce((obj, plugin) => {
                const fn = fp.get(method, plugin);
                return fn && fp.isFunction(fn) ? fn(obj, ...args) : obj;
            }, initial);
        };
    }
    static reduceFromPluginsAsync(plugins) {
        return (method, initial, ...args) => {
            return pReduce([...(plugins || [])], (obj, plugin) => {
                const fn = fp.get(method, plugin);
                return Promise.resolve(fn && fp.isFunction(fn) ? fn(obj, ...args) : obj);
            }, initial);
        };
    }
}
function createPlugin(factory) {
    return new Plugin(factory);
}

// TODO: type repo object returned from get-pkg-repo
const parseRepo = () => {
    try {
        const pkg = fs.readJsonSync(appPackageJson);
        return getPkgRepo(pkg);
    }
    catch (err) {
        return null;
    }
};
const getRepoUrl = () => {
    const repo = parseRepo();
    return (repo &&
        repo.browsetemplate &&
        repo.browsetemplate
            .replace('{domain}', repo.domain)
            .replace('{user}', repo.user)
            .replace('{project}', repo.project)
            .replace('{/tree/committish}', ''));
};
const getBitBucketPath = (branch, relative) => {
    const querystring = `?mode=edit&spa=0&at=${branch}&fileviewer=file-view-default`;
    const filepath = path.join(`/src/${branch}`, relative, `{{filepath}}`);
    return `${filepath}${querystring}`;
};
const getTree = (repo, branch, relative) => {
    const defaultPath = path.join(`/edit/${branch}`, relative, `{{filepath}}`);
    const bitBucketPath = getBitBucketPath(branch, relative);
    if (repo && repo.type === 'bitbucket')
        return bitBucketPath;
    return defaultPath;
};
const getRepoEditUrl = (src, branch) => {
    try {
        const repo = parseRepo();
        const project = path.parse(findup.sync('.git')).dir;
        const root$$1 = path.join(root, src);
        const relative = path.relative(project, root$$1);
        const tree = getTree(repo, branch, relative);
        return (repo &&
            repo.browsetemplate &&
            repo.browsetemplate
                .replace('{domain}', repo.domain)
                .replace('{user}', repo.user)
                .replace('{project}', repo.project)
                .replace('{/tree/committish}', tree));
    }
    catch (err) {
        return null;
    }
};

const fromTemplates = (file) => path.join(templates, file);
const pe = new PrettyError();
const mapToObj = (map) => Array.from(map.entries()).reduce((obj, [key, value]) => (Object.assign({}, obj, { [`${key}`]: value })), {});
const matchFilesWithSrc = (config) => (files) => {
    const src = path.relative(root, config.src);
    return files.map(file => (file.startsWith(src) ? file : path.join(src, file)));
};
const writeAppFiles = async (config, dev) => {
    const { plugins, theme } = config;
    const props = Plugin.propsOfPlugins(plugins);
    const onPreRenders = props('onPreRender');
    const onPostRenders = props('onPostRender');
    const root$$1 = await fs$2.compiled(fromTemplates('root.tpl.js'), { minimize: false });
    const js = await fs$2.compiled(fromTemplates('index.tpl.js'), { minimize: false });
    const websocketUrl = `ws://${config.websocketHost}:${config.websocketPort}`;
    const rawRootJs = root$$1({
        theme,
        isProd: !dev,
        wrapper: config.wrapper,
        hashRouter: config.hashRouter,
        websocketUrl,
    });
    const rawIndexJs = js({
        onPreRenders,
        onPostRenders,
        isProd: !dev,
    });
    await fs.remove(rootJs);
    await fs.remove(indexJs);
    await fs$2.touch(rootJs, rawRootJs);
    await fs$2.touch(indexJs, rawIndexJs);
};
class Entries {
    static async writeApp(config, dev) {
        await fs.ensureDir(app);
        await writeAppFiles(config, Boolean(dev));
    }
    constructor(config) {
        this.repoEditUrl = getRepoEditUrl(config.src, config.editBranch);
        this.all = new Map();
        this.get = async () => this.getMap(config);
    }
    async getMap(config) {
        const { src, files: pattern, ignore, plugins } = config;
        const arr = Array.isArray(pattern) ? pattern : [pattern];
        const toMatch = matchFilesWithSrc(config);
        const files = await glob(toMatch(arr), {
            ignore: ['**/node_modules/**'].concat(ignore),
            onlyFiles: true,
            unique: true,
            nocase: true,
            matchBase: true,
        });
        const createEntry = async (file) => {
            try {
                const ast = await mdast.parseMdx(file);
                const entry = new Entry(ast, file, src);
                if (this.repoEditUrl)
                    entry.setLink(this.repoEditUrl);
                const { settings } = entry, rest = tslib_1.__rest(entry, ["settings"]);
                return Object.assign({}, settings, rest);
            }
            catch (err) {
                console.log(err);
                config.debug && pe.render(err);
                return null;
            }
        };
        const reduce = Plugin.reduceFromPlugins(plugins);
        const modifiedFiles = reduce('modifyFiles', files);
        const map = new Map();
        const entries = await Promise.all(modifiedFiles.map(createEntry).filter(Boolean));
        for (const entry of entries) {
            if (entry) {
                map.set(entry.filepath, entry);
            }
        }
        this.all = map;
        return mapToObj(map);
    }
}

const onSignal = (cb) => {
    const signals = ['SIGINT', 'SIGTERM'];
    for (const sig of signals) {
        process.on(sig, async () => {
            await cb();
            process.exit();
        });
    }
};

const isSocketOpened = (socket) => socket.readyState === WS.OPEN;
const sender = (socket) => (type, payload) => {
    if (socket && isSocketOpened(socket)) {
        socket.send(JSON.stringify({ type, payload }));
    }
};
class DataServer {
    constructor(server, port, host) {
        this.states = new Set();
        this.state = {};
        if (server) {
            this.client = new WS.Server({
                server,
                port,
                host,
            });
        }
    }
    register(states) {
        for (const state of states)
            this.states.add(state);
        return this;
    }
    async init() {
        await Promise.all(Array.from(this.states).map(async (state) => {
            if (!fp.isFunction(state.init))
                return;
            return state.init({
                state: Object.assign({}, this.state),
                setState: this.setState(),
            });
        }));
        this.updateStateFile();
    }
    async listen() {
        if (this.client) {
            this.client.on('connection', socket => {
                const close = this.handleConnection(socket);
                const handleClose = async () => {
                    await close();
                    socket.terminate();
                };
                this.client.on('close', handleClose);
                onSignal(handleClose);
            });
        }
    }
    async close() {
        await Promise.all(Array.from(this.states).map(async (state) => fp.isFunction(state.close) &&
            state.close({
                state: Object.assign({}, this.state),
                setState: this.setState(),
            })));
    }
    handleConnection(socket) {
        const states = Array.from(this.states).map(async (state) => fp.isFunction(state.update) &&
            state.update({
                state: this.state,
                setState: this.setState(socket),
            }));
        return async () => {
            const fns = await Promise.all(states.filter(Boolean));
            for (const fn of fns)
                fp.isFunction(fn) && fn();
        };
    }
    setState(socket) {
        const send = sender(socket);
        return (key, val) => {
            this.state[key] = val;
            send(`state.${key}`, val);
            this.updateStateFile();
        };
    }
    async updateStateFile() {
        await fs.outputJSON(db, this.state);
    }
}

const toOmit = ['_', '$0', 'version', 'help'];
const htmlContext = {
    lang: 'en',
    favicon: 'https://cdn-std.dprcdn.net/files/acc_649651/LUKiMl',
};
const parseConfig = async (argv) => {
    const port = await detectPort(argv.port);
    const websocketPort = await detectPort(argv.websocketPort);
    const initial = fp.omit(toOmit, argv);
    const defaultConfig = Object.assign({}, initial, { port,
        websocketPort,
        htmlContext, menu: [], plugins: [], mdPlugins: [], hastPlugins: [], ignore: [
            'readme.md',
            'changelog.md',
            'code_of_conduct.md',
            'contributing.md',
            'license.md',
        ], themeConfig: {}, docgenConfig: {}, modifyBundlerConfig: (config) => config, modifyBabelRc: (babelrc) => babelrc, onCreateWebpackChain: () => null });
    const config = argv.config
        ? loadCfg.loadFrom(path.resolve(argv.config), defaultConfig)
        : loadCfg.load('docz', defaultConfig);
    const reduceAsync = Plugin.reduceFromPluginsAsync(config.plugins);
    return reduceAsync('setConfig', Object.assign({}, config, { paths }));
};

class Bundler {
    constructor(params) {
        const { args, config, server, build } = params;
        const run = Plugin.runPluginsMethod(args.plugins);
        this.args = args;
        this.config = config;
        this.server = server;
        this.builder = build;
        this.hooks = {
            onCreateWebpackChain(config, dev, args) {
                run('onCreateApp', config, dev, args);
            },
            onPreCreateApp(app$$1) {
                run('onPreCreateApp', app$$1);
            },
            onCreateApp(app$$1) {
                run('onCreateApp', app$$1);
            },
            onServerListening(server) {
                run('onServerListening', server);
            },
        };
    }
    async mountConfig(env) {
        const { plugins } = this.args;
        const isDev = env !== 'production';
        const reduce = Plugin.reduceFromPlugins(plugins);
        const userConfig = await this.config(this.hooks);
        const config = reduce('modifyBundlerConfig', userConfig, isDev, this.args);
        return this.args.modifyBundlerConfig(config, isDev, this.args);
    }
    async createApp(config) {
        return this.server(config, this.hooks);
    }
    async build(config) {
        const dist = getDist(this.args.dest);
        const publicDir = path.join(root, this.args.public);
        if (root === path.resolve(dist)) {
            logger__default.fatal(new Error('Unexpected option: "dest" cannot be set to the current working directory.'));
            process.exit(1);
        }
        await this.builder(config, dist, publicDir);
    }
}

const excludeNodeModules = (filepath) => /node_modules/.test(filepath);
const sourceMaps = (config, args) => {
    const srcPath = path.resolve(root, args.src);
    config.module
        .rule('sourcemaps')
        .test(/\.(js|mjs|jsx|ts|tsx|md|mdx)$/)
        .include.add(srcPath)
        .add(app)
        .end()
        .use('sourcemaps')
        .loader(require.resolve('source-map-loader'))
        .end()
        .enforce('pre');
};
const addScriptLoaders = (rule, babelrc, args) => rule
    .when(!args.debug, rule => rule
    .use('cache-loader')
    .loader(require.resolve('cache-loader'))
    .options({
    cacheDirectory: cache,
}))
    .use('thread-loader')
    .loader(require.resolve('thread-loader'))
    .options({
    workers: require('os').cpus().length - 1,
})
    .end()
    .use('babel-loader')
    .loader(require.resolve('babel-loader'))
    .options(babelrc)
    .end();
const js = (config, args, babelrc) => {
    const srcPath = path.resolve(root, args.src);
    const rule = config.module
        .rule('js')
        .test(/\.(js|mjs|jsx)$/)
        .include.add(srcPath)
        .add(root)
        .add(app)
        .end()
        .exclude.add(excludeNodeModules)
        .end();
    addScriptLoaders(rule, babelrc, args);
};
const ts = (config, args, babelrc) => {
    const srcPath = path.resolve(root, args.src);
    const rule = config.module
        .rule('ts')
        .test(/\.(ts|tsx?)$/)
        .include.add(srcPath)
        .add(root)
        .add(app)
        .end()
        .exclude.add(excludeNodeModules)
        .end();
    addScriptLoaders(rule, babelrc, args);
};
const mdx = (config, args, babelrc) => {
    const { mdPlugins, hastPlugins } = args;
    const srcPath = path.resolve(root, args.src);
    const rule = config.module
        .rule('mdx')
        .test(/\.(md|markdown|mdx)$/)
        .include.add(srcPath)
        .add(root)
        .end()
        .exclude.add(excludeNodeModules)
        .end();
    addScriptLoaders(rule, babelrc, args)
        .use('mdx-loader')
        .loader(require.resolve('@mdx-js/loader'))
        .options({
        type: 'yaml',
        marker: '-',
        mdPlugins: mdPlugins.concat([matter, remarkDocz]),
        hastPlugins: hastPlugins.concat([
            rehypeDocz(root, args.codeSandbox),
            slug,
        ]),
    });
};
const INLINE_LIMIT = 10000;
const images = (config) => {
    config.module
        .rule('images')
        .test(/\.(png|jpe?g|gif)(\?.*)?$/)
        .use('url-loader')
        .loader(require.resolve('url-loader'))
        .options({
        limit: INLINE_LIMIT,
        name: `static/img/[name].[hash:8].[ext]`,
    });
};
const svg = (config) => {
    config.module
        .rule('svg')
        .test(/\.(svg)(\?.*)?$/)
        .use('file-loader')
        .loader(require.resolve('file-loader'))
        .options({
        name: `static/img/[name].[hash:8].[ext]`,
    });
};
const media = (config) => {
    config.module
        .rule('media')
        .test(/\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/)
        .use('url-loader')
        .loader(require.resolve('url-loader'))
        .options({
        limit: INLINE_LIMIT,
        name: `static/media/[name].[hash:8].[ext]`,
    });
};
const fonts = (config) => {
    config.module
        .rule('fonts')
        .test(/\.(woff2?|eot|ttf|otf)(\?.*)?$/i)
        .use('url-loader')
        .loader(require.resolve('url-loader'))
        .options({
        limit: INLINE_LIMIT,
        name: `static/fonts/[name].[hash:8].[ext]`,
    });
};

const wrapItems = (item) => Object.keys(item)
    .map(key => `${key}="${item[key]}"`)
    .join(' ');
const generateTags = (template) => (items = []) => items.map(template).join('');
const generateMetaTags = generateTags(item => `<meta ${wrapItems(item)}>`);
const generateLinkTags = generateTags(item => `<link ${wrapItems(item)}>`);
const generateScriptTags = generateTags(item => `<script ${wrapItems(item)}></script>`);
const generateRawTags = (items = []) => {
    if (typeof items === 'string' || items instanceof String)
        return items;
    return items.map(item => item).join('');
};
const getHtmlFilepath = (indexHtml$$1) => indexHtml$$1
    ? path.resolve(root, indexHtml$$1)
    : fromTemplates('index.tpl.html');
const getPublicUrl = (config, dev) => {
    const prefix = config.base === '/' ? '' : config.base;
    return dev ? prefix : `${prefix}/public`;
};
const emptyLineTrim = new ctags.TemplateTag(ctags.replaceResultTransformer(/^\s*[\r\n]/gm, ''), ctags.trimResultTransformer);
const htmlTemplate = async (indexHtml$$1) => fs$2.compiled(getHtmlFilepath(indexHtml$$1), {
    minimize: false,
    escape: false,
});
const parseHtml = ({ config, ctx, dev, template }) => {
    const { title, description } = config;
    const { publicPath, css, js, lang = 'en', favicon, head = [], body = [], trimWhitespace, } = ctx;
    const headStr = `
    ${favicon ? `<link rel="icon" type="image/x-icon" href="${favicon}">` : ''}
    ${head.meta ? generateMetaTags(head.meta) : ''}
    ${head.links ? generateLinkTags(head.links) : ''}
    ${head.raw ? generateRawTags(head.raw) : ''}
    ${head.scripts ? generateScriptTags(head.scripts) : ''}
    ${miniHtmlWebpack.generateCSSReferences(css, publicPath)}`;
    const footerStr = `
    ${body.raw ? generateRawTags(body.raw) : ''}
    ${body.scripts ? generateScriptTags(body.scripts) : ''}
    ${miniHtmlWebpack.generateJSReferences(js, publicPath)}`;
    const doc = ctags.html(template({
        title,
        description,
        lang,
        head: headStr,
        footer: footerStr,
        publicUrl: getPublicUrl(config, dev),
    }));
    return trimWhitespace ? ctags.oneLineTrim(doc) : emptyLineTrim(doc);
};

const assets = (config, args, env) => {
    const isProd = env === 'production';
    const base = servedPath(args.base);
    const publicPath = isProd ? base : '/';
    config.plugin('assets-plugin').use(manifestPlugin, [
        {
            publicPath,
            fileName: 'assets.json',
        },
    ]);
};
const analyzer = (config) => {
    config.plugin('bundle-analyzer').use(webpackBundleAnalyzer.BundleAnalyzerPlugin, [
        {
            generateStatsFile: true,
            openAnalyzer: false,
            analyzerMode: 'static',
        },
    ]);
};
const friendlyErrors = (config, { host, port }) => {
    const isLocalhost = host === '127.0.0.1' || host === '0.0.0.0';
    const hostname = isLocalhost ? 'localhost' : host;
    config.plugin('friendly-errors').use(friendlyErrorsPlugin, [
        {
            compilationSuccessInfo: {
                messages: [`Your application is running at http://${hostname}:${port}`],
            },
        },
    ]);
};
const injections = (config, args, env) => {
    const { stringify } = JSON;
    const base = servedPath(args.base);
    const plugin = require('webpack/lib/DefinePlugin');
    const websocketUrl = `ws://${args.websocketHost}:${args.websocketPort}`;
    const isDev = env !== 'production';
    config.plugin('injections').use(plugin, [
        Object.assign({}, getClientEnvironment(base).stringified, { NODE_ENV: stringify(env), DOCZ_BASE_URL: args.hashRouter ? stringify('/') : stringify(base), DOCZ_WEBSOCKET_URL: stringify(isDev && websocketUrl), DOCZ_HASH_ROUTER: stringify(args.hashRouter || false) }),
    ]);
};
const ignore = (config) => {
    config
        .plugin('ignore-plugin')
        .use(webpack.IgnorePlugin, [
        /(regenerate\-unicode\-properties)|(elliptic)/,
        /node_modules/,
    ]);
};
const hot = (config) => {
    config
        .plugin('hot-module-replacement')
        .use(webpack.HotModuleReplacementPlugin, [
        {
            multiStep: true,
        },
    ]);
};
const html = async (config, args, env) => {
    const dev = env !== 'production';
    const template = await htmlTemplate(args.indexHtml);
    config.plugin('html-plugin').use(miniHtmlWebpack__default, [
        {
            context: Object.assign({}, args.htmlContext, { trimWhitespace: true }),
            template: (ctx) => {
                const doc = parseHtml({ ctx, dev, template, config: args });
                return dev
                    ? doc
                    : htmlMinifier.minify(doc, {
                        removeComments: true,
                        collapseWhitespace: true,
                        removeRedundantAttributes: true,
                        useShortDoctype: true,
                        removeEmptyAttributes: true,
                        removeStyleLinkTypeAttributes: true,
                        keepClosingSlash: true,
                        minifyJS: true,
                        minifyCSS: true,
                        minifyURLs: true,
                    });
            },
        },
    ]);
};
const webpackBar = (config, args) => {
    config.plugin('webpackbar').use(webpackBarPlugin, [
        {
            name: 'Docz',
            color: '#41b883',
        },
    ]);
};
const watchNodeModulesPlugin = (config) => {
    config
        .plugin('watch-missing-node-modules')
        .use(watchMissingNodeModules, [appNodeModules]);
};

const minifier = (config, args) => {
    config.optimization.minimizer('js').use(TerserPlugin, [
        {
            terserOptions: {
                parse: {
                    ecma: 8,
                },
                compress: {
                    ecma: 5,
                    warnings: false,
                    comparisons: false,
                },
                mangle: {
                    safari10: true,
                },
                output: {
                    ecma: 5,
                    comments: false,
                    ascii_only: true,
                },
            },
            parallel: true,
            cache: !args.debug,
            sourceMap: args.sourcemaps,
        },
    ]);
};

const getBabelConfig = async (args, env, typescript) => {
    const isProd = env === 'production';
    const isDev = env === 'development';
    const localBabelRc = loadCfg.load('babel', { presets: [], plugins: [] }, false, true);
    const presets = [
        [
            require.resolve('babel-preset-react-app'),
            {
                typescript,
                flow: !args.typescript,
            },
        ],
    ];
    const defaultPlugins = [
        require.resolve('babel-plugin-export-metadata'),
        [
            require.resolve('babel-plugin-named-asset-import'),
            {
                loaderMap: {
                    svg: {
                        ReactComponent: '@svgr/webpack?-prettier,-svgo![path]',
                    },
                },
            },
        ],
    ];
    const config = merge(localBabelRc, {
        presets,
        babelrc: false,
        cacheCompression: args.debug ? false : isProd,
        cacheDirectory: !args.debug,
        cacheIdentifier: args.debug
            ? null
            : getCacheIdentifier(isProd ? 'production' : isDev && 'development', [
                'docz',
                'docz-core',
            ]),
        compact: isProd,
        customize: require.resolve('babel-preset-react-app/webpack-overrides'),
        plugins: defaultPlugins.concat(!isProd ? [require.resolve('react-hot-loader/babel')] : []),
    });
    const reduce = Plugin.reduceFromPlugins(args.plugins);
    const newConfig = reduce('modifyBabelRc', config, args);
    return args.modifyBabelRc(newConfig, args);
};

const createConfig = (args, env) => async (hooks) => {
    const { debug } = args;
    const config = new Config();
    const isProd = env === 'production';
    const base = servedPath(args.base);
    const dist = getDist(args.dest);
    const srcPath = path.resolve(root, args.src);
    const publicPath = isProd ? base : '/';
    /**
     * general
     */
    config.context(root);
    config.set('mode', env);
    config.when(args.sourcemaps, cfg => cfg.devtool(isProd ? 'source-map' : 'cheap-module-eval-source-map'), cfg => cfg.devtool(false));
    config.node.merge({
        child_process: 'empty',
        dgram: 'empty',
        fs: 'empty',
        net: 'empty',
        tls: 'empty',
    });
    /**
     * output
     */
    const outputProd = (output) => output
        .filename('static/js/[name].[hash].js')
        .sourceMapFilename('static/js/[name].[hash].js.map')
        .chunkFilename('static/js/[name].[chunkhash:8].js');
    const outputDev = (output) => output
        .filename('static/js/[name].js')
        .sourceMapFilename('static/js/[name].js.map');
    config.output
        .pathinfo(true)
        .path(path.resolve(root, dist))
        .publicPath(publicPath)
        .when(isProd, outputProd, outputDev)
        .crossOriginLoading('anonymous')
        .devtoolModuleFilenameTemplate((info) => path.resolve(info.resourcePath).replace(/\\/g, '/'));
    /**
     * entries
     */
    config
        .entry('app')
        .add(require.resolve('react-dev-utils/webpackHotDevClient'))
        .add(require.resolve('@babel/polyfill'))
        .add(indexJs);
    /**
     * resolve
     */
    config.resolve.set('symlinks', true);
    config.resolve.extensions
        .add('.web.js')
        .add('.mjs')
        .add('.js')
        .add('.json')
        .add('.web.jsx')
        .add('.jsx')
        .add('.mdx')
        .end();
    config.resolve.alias.set('~db', db);
    config.resolve.alias.set('~imports', importsJs);
    config.resolve.alias.set('react-native$', 'react-native-web');
    const inYarnWorkspaces = __dirname.includes('/docz/core/docz-core');
    const doczDependenciesDir = inYarnWorkspaces
        ? path.join(__dirname, '../../../../node_modules')
        : ownNodeModules;
    config.resolve.modules
        .add('node_modules')
        .add(doczDependenciesDir)
        .add(srcPath)
        .add(root)
        .merge(envDotProp.get('node.path')
        .split(path.delimiter)
        .filter(Boolean));
    config.resolveLoader
        .set('symlinks', true)
        .modules // prioritize our own
        .add('node_modules')
        .add(doczDependenciesDir)
        .add(root);
    /**
     * loaders
     */
    const jsBabelRc = await getBabelConfig(args, env);
    const tsBabelRc = await getBabelConfig(args, env, true);
    config.when(args.sourcemaps, cfg => sourceMaps(cfg, args));
    js(config, args, jsBabelRc);
    mdx(config, args, jsBabelRc);
    images(config);
    svg(config);
    media(config);
    fonts(config);
    await html(config, args, env);
    assets(config, args, env);
    ignore(config);
    injections(config, args, env);
    hot(config);
    config.when(debug, cfg => analyzer(cfg));
    config.when(!isProd, cfg => watchNodeModulesPlugin(cfg));
    config.when(!debug && !isProd, cfg => {
        webpackBar(cfg, args);
        friendlyErrors(cfg, args);
    });
    /**
     * typescript setup
     */
    config.when(args.typescript, cfg => {
        cfg.resolve.extensions
            .prepend('.ts')
            .prepend('.tsx')
            .end();
        ts(cfg, args, tsBabelRc);
    });
    /**
     * optimization
     */
    config.optimization
        .nodeEnv(env)
        .namedModules(true)
        .minimize(isProd)
        .splitChunks({
        cacheGroups: {
            vendor: {
                test: /[\\/]node_modules[\\/]/,
                name: 'vendors',
                chunks: 'all',
            },
        },
    });
    config.performance.hints(false);
    config.when(isProd, cfg => minifier(cfg, args));
    hooks.onCreateWebpackChain(config, !isProd, args);
    args.onCreateWebpackChain(config, !isProd, args);
    return config.toConfig();
};

const devServerConfig = (hooks, args) => {
    const srcPath = path.resolve(root, args.src);
    const publicDirPath = path.resolve(root, args.public);
    const publicDir = path.join(root, publicDirPath);
    const nonExistentDir = path.resolve(__dirname, 'non-existent');
    return {
        publicPath: '/',
        compress: true,
        logLevel: args.debug ? 'debug' : 'silent',
        clientLogLevel: args.debug ? 'info' : 'none',
        contentBase: [nonExistentDir],
        watchContentBase: true,
        hot: true,
        quiet: !args.debug,
        open: true,
        watchOptions: {
            ignored: ignoredFiles(srcPath),
        },
        overlay: false,
        host: args.host,
        port: args.port,
        historyApiFallback: {
            disableDotRule: true,
        },
        before(app$$1, server) {
            app$$1.use('/public', express.static(publicDir));
            app$$1.use(evalSourceMapMiddleware(server));
            app$$1.use(errorOverlayMiddleware());
            hooks.onPreCreateApp(app$$1);
        },
        after(app$$1) {
            hooks.onCreateApp(app$$1);
        },
    };
};

const pe$1 = new PrettyError();
const createCompiler = (config) => new Promise(resolve$$1 => {
    try {
        resolve$$1(require('webpack')(config));
    }
    catch (err) {
        logger__default.fatal(chalk.red('Failed to compile.'));
        pe$1.render(err);
        process.exit(1);
    }
});
const server = (args) => async (config, hooks) => ({
    start: async () => {
        const serverConfig = devServerConfig(hooks, args);
        const compiler = await createCompiler(config);
        const devServer = new WebpackDevServer(compiler, serverConfig);
        return devServer.listen(args.port, args.host, err => {
            if (err)
                return logger__default.fatal(err);
            hooks.onServerListening(devServer);
        });
    },
});

const FSR = require('react-dev-utils/FileSizeReporter');
const formatWebpackMessages = require('react-dev-utils/formatWebpackMessages');
const { measureFileSizesBeforeBuild, printFileSizesAfterBuild } = FSR;
const WARN_AFTER_BUNDLE_GZIP_SIZE = 512 * 1024;
const WARN_AFTER_CHUNK_GZIP_SIZE = 1024 * 1024;
const hasCiEnvVar = () => envDotProp.get('ci', false, { parse: true });
const copyPublicFolder = async (dest, publicDir) => {
    if (await fs.pathExists(publicDir)) {
        await fs.copy(publicDir, distPublic(dest), {
            dereference: true,
            filter: file => file !== indexHtml,
        });
    }
};
const compile = (config) => new Promise((resolve$$1, reject) => {
    let compiler;
    try {
        compiler = webpack__default(config);
    }
    catch (err) {
        onError(err);
    }
    compiler &&
        compiler.run((err, stats) => {
            if (err)
                reject(err);
            resolve$$1(stats);
        });
});
const builder = async (config, previousFileSizes) => new Promise(async (resolve$$1, reject) => {
    try {
        const stats = await compile(config);
        const messages = formatWebpackMessages(stats.toJson({}, true));
        if (messages.errors.length) {
            return reject(new Error(messages.errors.join('\n\n')));
        }
        if (hasCiEnvVar() && messages.warnings.length) {
            logger.warn('\nTreating warnings as errors because process.env.CI = true.\n' +
                'Most CI servers set it automatically.\n');
            return reject(new Error(messages.warnings.join('\n\n')));
        }
        return resolve$$1({
            stats,
            previousFileSizes,
            warnings: messages.warnings,
        });
    }
    catch (err) {
        reject(err);
    }
});
const onSuccess = (dist, { stats, previousFileSizes, warnings }) => {
    if (warnings.length) {
        logger.log();
        logger.warn('Compiled with warnings.\n');
        logger.warn(warnings.join('\n\n'));
        logger.warn('\nSearch for the ' +
            chalk.underline(chalk.yellow('keywords')) +
            ' to learn more about each warning.');
        logger.warn('To ignore, add ' +
            chalk.cyan('// eslint-disable-next-line') +
            ' to the line before.\n');
    }
    logger.log();
    logger.log(`File sizes after gzip:\n`);
    printFileSizesAfterBuild(stats, previousFileSizes, dist, WARN_AFTER_BUNDLE_GZIP_SIZE, WARN_AFTER_CHUNK_GZIP_SIZE);
    logger.log();
};
const onError = (err) => {
    logger.log();
    logger.fatal(err);
    process.exit(1);
    logger.log();
};
const build = async (config, dist, publicDir) => {
    const interactive = new logger.Signale({ interactive: true, scope: 'build' });
    try {
        interactive.start('Creating an optimized bundle');
        await fs.ensureDir(dist);
        const previousFileSizes = await measureFileSizesBeforeBuild(dist);
        await fs.emptyDir(dist);
        await copyPublicFolder(dist, publicDir);
        const result = await builder(config, previousFileSizes);
        interactive.success('Build successfully created');
        onSuccess(dist, result);
    }
    catch (err) {
        logger.fatal(chalk.red('Failed to compile.\n'));
        onError(err);
    }
};

const bundler = (args, env) => new Bundler({
    args,
    build,
    config: createConfig(args, env),
    server: server(args),
});

const writeImports = async (map) => {
    const imports = await fs$2.compiled(fromTemplates('imports.tpl.js'));
    const rawImportsJs = imports({ entries: Object.values(map) });
    await fs$2.touch(path.join(app, 'imports.js'), rawImportsJs);
};
const updateEntries = (entries) => async (p) => {
    const old = p.state.entries;
    const map = await entries.get();
    if (map && !equal(old, map)) {
        await writeImports(map);
        p.setState('entries', map);
    }
};
const state = (entries, config) => {
    const src = path.relative(root, config.src);
    const files = path.join(src, config.files);
    const watcher = chokidar.watch(files, {
        cwd: root,
        ignored: /(((^|[\/\\])\..+)|(node_modules))/,
        persistent: true,
    });
    watcher.setMaxListeners(Infinity);
    return {
        id: 'entries',
        init: updateEntries(entries),
        close: () => watcher.close(),
        update: async (params) => {
            const update = updateEntries(entries);
            watcher.on('change', async () => update(params));
            watcher.on('unlink', async () => update(params));
            watcher.on('raw', async (event, path$$1, details) => {
                if (details.event === 'moved' && details.type === 'directory') {
                    await update(params);
                }
            });
            return () => watcher.close();
        },
    };
};

const getInitialConfig = (config) => {
    const pkg = fs.readJsonSync(appPackageJson, { throws: false });
    const repoUrl = getRepoUrl();
    return {
        title: config.title,
        description: config.description,
        menu: config.menu,
        ordering: config.ordering,
        version: get(pkg, 'version'),
        repository: repoUrl,
        native: config.native,
        codeSandbox: config.codeSandbox,
        themeConfig: config.themeConfig,
    };
};
const updateConfig = (config) => async ({ setState }) => setState('config', loadCfg.load('docz', getInitialConfig(config), true, false));
const state$1 = (config) => {
    const watcher = chokidar.watch(loadCfg.finds('docz'), {
        cwd: root,
        ignored: /(((^|[\/\\])\..+)|(node_modules))/,
        persistent: true,
    });
    watcher.setMaxListeners(Infinity);
    return {
        id: 'config',
        init: updateConfig(config),
        close: () => watcher.close(),
        update: async (params) => {
            const update = updateConfig(config);
            const fn = async () => update(params);
            watcher.on('add', fn);
            watcher.on('change', fn);
            watcher.on('unlink', fn);
            return () => watcher.close();
        },
    };
};

const pe$2 = new PrettyError();
const fileFullPath = (filepath) => path.join(root, filepath);
const tsParser = async (files, config) => {
    const tsConfigPath = await findup('tsconfig.json', { cwd: root });
    if (!tsConfigPath)
        return {};
    try {
        const { parse } = reactDocgenTs.withCustomConfig(tsConfigPath, {
            propFilter(prop, component) {
                if (prop.parent == null)
                    return true;
                const propFilter = config.docgenConfig.propFilter;
                const val = propFilter && fp.isFunction(propFilter) && propFilter(prop);
                return !prop.parent.fileName.includes('node_modules') || Boolean(val);
            },
        });
        return files
            .map(filepath => ({ [fileFullPath(filepath)]: parse(filepath) }))
            .reduce((obj, val) => (Object.assign({}, obj, val)), {});
    }
    catch (err) {
        logger__default.fatal('Error parsing static types.');
        pe$2.render(err);
        return {};
    }
};
const jsParser = (files, config) => {
    const resolver = config.docgenConfig.resolver ||
        reactDocgen.resolver.findAllExportedComponentDefinitions;
    return files.reduce((memo, filepath) => {
        const handlers = reactDocgen.defaultHandlers.concat([
            externalProptypesHandler(filepath),
            importedProptypesHandler(filepath),
            actualNameHandler,
        ]);
        const code = fs.readFileSync(filepath, 'utf-8');
        try {
            const data = reactDocgen.parse(code, resolver, handlers);
            memo[fileFullPath(filepath)] = data;
        }
        catch (err) {
            pe$2.render(err);
        }
        return memo;
    }, {});
};
const docgen = async (files, config) => config.typescript ? tsParser(files, config) : jsParser(files, config);

const getPattern = (ts) => {
    const tsPattern = '**/*.{ts,tsx}';
    const jsPattern = '**/*.{js,jsx,mjs}';
    return [ts ? tsPattern : jsPattern, '!**/node_modules'];
};
const initial = (config) => async (p) => {
    const pattern = getPattern(config.typescript);
    const files = await glob(pattern, { cwd: root });
    const metadata = await docgen(files, config);
    p.setState('props', metadata);
};
const add = (config) => (p) => async (filepath) => {
    const old = fp.get('state.props', p);
    const metadata = await docgen([filepath], config);
    if (metadata && !equal(old, metadata)) {
        p.setState('props', Object.assign({}, old, metadata));
    }
};
const remove = (config) => (p) => async (filepath) => p.setState('props', fp.omit(filepath, fp.get('state.props', p)));
const state$2 = (config) => {
    const pattern = getPattern(config.typescript);
    const watcher = chokidar.watch(pattern, {
        cwd: root,
        ignored: /(((^|[\/\\])\..+)|(node_modules))/,
        persistent: true,
    });
    watcher.setMaxListeners(Infinity);
    return {
        id: 'props',
        init: initial(config),
        close: () => watcher.close(),
        update: async (params) => {
            const addFilepath = add(config);
            const removeFilepath = remove(config);
            watcher.on('add', addFilepath(params));
            watcher.on('change', addFilepath(params));
            watcher.on('unlink', removeFilepath(params));
            return () => watcher.close();
        },
    };
};

process.setMaxListeners(Infinity);
const pe$3 = new PrettyError();
const dev = async (args) => {
    const env = envDotProp.get('node.env');
    const config = await parseConfig(args);
    const bundler$$1 = bundler(config, env);
    const entries = new Entries(config);
    const bundlerConfig = await bundler$$1.mountConfig(env);
    const app = await bundler$$1.createApp(bundlerConfig);
    try {
        await Entries.writeApp(config, true);
    }
    catch (err) {
        logger.fatal('Failed to build your files');
        pe$3.render(err);
        process.exit(1);
    }
    const server = await app.start();
    const dataServer = new DataServer(server, config.websocketPort, config.websocketHost);
    if (args.propsParser)
        dataServer.register([state$2(config)]);
    dataServer.register([state$1(config), state(entries, config)]);
    try {
        await dataServer.init();
        await dataServer.listen();
    }
    catch (err) {
        logger.fatal('Failed to process data server');
        pe$3.render(err);
        await dataServer.close();
        process.exit(1);
    }
    onSignal(async () => {
        await dataServer.close();
        server.close();
    });
    server.on('close', async () => {
        await dataServer.close();
    });
};

const pe$4 = new PrettyError();
const build$1 = async (args) => {
    const env = envDotProp.get('node.env');
    const config = await parseConfig(args);
    const entries = new Entries(config);
    const bundler$$1 = bundler(config, env);
    const bundlerConfig = await bundler$$1.mountConfig(env);
    const run = Plugin.runPluginsMethod(config.plugins);
    const dataServer = new DataServer();
    if (args.propsParser)
        dataServer.register([state$2(config)]);
    dataServer.register([state$1(config), state(entries, config)]);
    try {
        await Entries.writeApp(config, true);
        await dataServer.init();
        await run('onPreBuild', config);
        await bundler$$1.build(bundlerConfig);
        await run('onPostBuild', config);
        await dataServer.close();
    }
    catch (err) {
        pe$4.render(err);
        process.exit(1);
    }
};

const cli = () => {
    return yargs.command('dev', 'initialize docz dev server', setArgs, async (args) => {
        setEnv('development');
        await dev(args);
    })
        .command('build', 'build dir as static site', setArgs, async (args) => {
        setEnv('production');
        await build$1(args);
        process.exit();
    })
        .demandCommand()
        .help()
        .wrap(72)
        .epilog('for more information visit https://github.com/pedronauck/docz')
        .showHelpOnFail(false, 'whoops, something went wrong! run with --help').argv;
};

/** cli exports */

exports.cli = cli;
exports.parseConfig = parseConfig;
exports.Plugin = Plugin;
exports.createPlugin = createPlugin;
exports.DataServer = DataServer;
exports.Entries = Entries;
exports.Entry = Entry;
